# -*- encoding: utf-8 -*-
##############################################################################
# Copyright (c) 2011 OpenERP Venezuela (http://openerp.com.ve)
# All Rights Reserved.
# Programmed by: Israel Fermín Montilla  <israel@openerp.com.ve>
#                Moisés López <moylop260@gmail.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################

import os
import ctypes
import glob
import serial
import time
import platform
import sys
import pygtk
pygtk.require(2.0)
import gtk

def get_port(port=False):
    devices = []
    if not port:
        if os.name == 'nt':
            for i in range(15):
              try:
                s = serial.Serial('COM%d'%i)
                devices.append( s.portstr )
                s.close()
              except:
                pass
        else:
            ports = glob.glob('/dev/ttyUSB*') + glob.glob('/dev/ttyS*')
            for port in ports:
                port_obj = serial.Serial(
                    port = port,
                    baudrate = 9600,
                    bytesize = serial.EIGHTBITS,
                    parity = serial.PARITY_EVEN,
                    stopbits = serial.STOPBITS_ONE,
                    timeout = 0.5,
                    writeTimeout = 1.2,
                    xonxoff = 0,
                    rtscts = 0
                )
                if port_obj.isOpen():
                    devices.append( port )
                port_obj.close()
    else:
        devices.append( port )
    return devices

def get_driver(fname_driver=False):
    if not fname_driver:
        if os.name == 'posix':
            architecture = platform.architecture()
            if architecture[0] == '32bit':
                fname_driver = os.path.join(os.path.expanduser("~/.pyfiscal/lib"), "lib300fi32.so")
            elif architecture[0] == '64bit':
                fname_driver = os.path.join(os.path.expanduser("~/.pyfiscal/lib"), "lib300fi64.so")
        elif os.name == 'nt':
            fname_driver = "TODO"
    return fname_driver

def render_error(message):
    error = ""
    if message[0] == "" or message[1] == "":
        error = "".join(message)
    else:
        error = ": ".join(message)
    md = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENTS,
            gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, error)
    md.run()

def translate_error(ack, st_1, st_2):
    if ack != 6:
        render("error de comunicación")
    elif st_1 == st_2 == 0:
        pass
    else:
        st_1_vals = {
                0: "",
                1: "Número de argumentos inválido",
                2: "Ticket abierto",
                4: "Comando inexistente",
                8: "Primer dato del comando no fue ESC (1BH)",
                16: "Impresora en error",
                32: "Error en el reloj",
                64: "Poco papel",
                128: "Fin de papel",
            }
        st_2_vals = {
                0: "",
                1: "Comando no ejecutado",
                2: "Identificación fiscal no programada",
                4: "Anulación no permitida",
                8: "Capacidad de tasas programables llena",
                16: "Tasa no programada",
                32: "Error en memoria RAM CMOS no volátil",
                64: "Memoria fiscal llena",
                128: "Tipo de parámetro inválido",
            }
        msg = (st_1_vals[st_1], st_2_val2[st_2])
        render_error(msg)

def exec_driver(route, port=False):
    driver_pytxt = """
import os
import ctypes
import glob
import serial
import time
import platform
import ctypes
""" + open(route,"r").read()

    ports = get_port()
    fname_driver = get_driver()
    port = ports and ports[0] or ''
    driver_obj = ctypes.cdll.LoadLibrary(fname_driver)
    ack_st = ctypes.c_uint * 4
    ack_st = ack_st(0, 0, 0, 0)
    exec driver_pytxt in {'port': port, 'fname_driver': fname_driver, 'driver_obj': driver_obj, 'ack_st':ack_st}
    translate_error(ack_st[0], ack_st[1], ack_st[2])

if __name__ == '__main__':
    script, route = sys.argv
    exec_driver(route)
