# -*- encoding: utf-8 -*-
##############################################################################
# Copyright (c) 2011 OpenERP Venezuela (http://openerp.com.ve)
# All Rights Reserved.
# Programmed by: Israel Fermín Montilla  <israel@openerp.com.ve>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################
from osv import osv
from osv import fields
import netsvc
import base64
from tools.translate import _

class wizard_pos_fp_bematech_4000fi_sale(osv.osv_memory):
    _name = "wizard.print.invoice.pos"

    def check_driver(self, cr, uid, ids, context):
        pass

    def update_printed(self, cr, uid, ids, context=None):
        order_obj = self.pool.get(context['active_model'])
        order = order_obj.browse(cr, uid, context['active_id'])
        if not order.printed:
            order.printed = True
        res = order_obj.write(cr, uid, context['active_id'], {'printed': order.printed})
        return {'type': 'ir.actions.act_window_close'}

    def _check_tax_mapping(self, cr, uid, tax):
        tax_id = tax[0].id
        client_obj = self.pool.get('fiscal.printing.client')
        client = client_obj.browse(cr, uid, client_obj.search(cr, uid, [('user_id', '=', uid)]))[0]
        valid_taxes = [t for t in client.tax_code_ids if t.tax_id.id == tax_id]
        if valid_taxes == []:
            raise osv.except_osv(_("ERROR !"), _("You don't have this tax code configured on this client. Go to Administration -> Customization -> Low Level Objects -> Fiscal Printers -> Fiscal Printing Client Configuration"))
        elif len(valid_taxes) > 1:
            raise osv.except_osv(_("ERROR !"), _("You more than one tax code configured for this case. Go to Administration -> Customization -> Low Level Objects -> Fiscal Printers -> Fiscal Printing Client Configuration"))
        else:
            return True

    def _check_invoice_data(self, cr, uid, context):
        orders = self.pool.get(context['active_model']).browse(cr, uid, context['active_ids'])
        for order in orders:
            lines = order.lines
            p_tmplts = [l.product_id.product_tmpl_id for l in lines]
            for t in p_tmplts:
                if len(t.taxes_id) > 1:
                    raise osv.except_osv(_("ERROR !"), _("You can sell products with just one configured tax, check the product account properties"))
                elif len(t.taxes_id) > 0:
                    res = self._check_tax_mapping(cr, uid, t.taxes_id)
        return True

    def _create_report(self, cr, uid, context):
        driver_name = self._get_driver(cr, uid, context) or False
        if not driver_name:
            raise osv.except_osv(_('Error !'), _('There is no driver configured for this cashier. Please, go to Administration -> Customization -> Low Level Objects -> Fiscal Printers.'))
        printed = self.pool.get(context['active_model']).browse(cr, uid, context['active_id']).printed
        if printed:
            raise osv.except_osv(_("ERROR !"), _("This ticket is already printed, call your manager if you really need to print it again"))
        report_name = "pos_fp_%s.%s_sale" % (driver_name, driver_name)
        service = netsvc.LocalService("report."+report_name)
        if self._check_invoice_data(cr, uid, context):
            (result, format) = service.create(cr, uid, context['active_ids'], {'model': context['active_model']}, {})
            return base64.encodestring(result)
        return None

    def _get_driver(self, cr, uid, ids):
        obj = self.pool.get('fiscal.printing.client').browse(cr, uid, self.pool.get('fiscal.printing.client').search(cr, uid, [('user_id', '=', uid)]))
        try:
            driver_name = obj[0].driver
        except Exception, e:
            raise osv.except_osv(_('Error !'), _('There is no driver configured for this cashier. Please, go to Administration -> Customization -> Low Level Objects -> Fiscal Printers.'))
        report_name = "pos_fp_%s.%s_sale" % (driver_name, driver_name)
        return driver_name

    _columns = {
            'driver': fields.char("Driver", 64, readonly=True),
            'report_code.pyfiscal': fields.binary("Report", readonly=True),
        }

    _defaults = {
            'driver': _get_driver,
            'report_code.pyfiscal': _create_report,
        }
wizard_pos_fp_bematech_4000fi_sale()
